import { input } from '@inquirer/prompts';
import chalk from 'chalk';
import { resolve } from 'path';
import { homedir } from 'os';
import ora from 'ora';
import { ChatStreamer } from './ChatStreamer.js';
import { FileUploader } from './FileUploader.js';
import { ConversationManager } from './ConversationManager.js';
import { loadGrokConfig, saveGrokConfig } from '../state/ConfigGrok.js';
// Controls the lifecycle and interactions of a Grok chat session
export class ChatController {
    page;
    commandRegistry;
    menuHandler;
    configDir;
    chatActive = false;
    cwd = process.cwd();
    fileIds = [];
    conversationId;
    enabledTool = null;
    chatStreamer;
    grokConfig;
    initialParentResponseId = '';
    fileUploader;
    conversationManager;
    constructor(page, commandRegistry, menuHandler, initialConversationId, configDir) {
        this.page = page;
        this.commandRegistry = commandRegistry;
        this.menuHandler = menuHandler;
        this.configDir = configDir;
        this.conversationId = initialConversationId;
        this.fileUploader = new FileUploader(page, this.fileIds, this.cwd); // Pass cwd to FileUploader
        this.conversationManager = new ConversationManager(page);
        this.chatStreamer = new ChatStreamer(page, this.conversationId, this.initialParentResponseId);
        this.grokConfig = {};
    }
    // Initiates the chat session with browser setup and user interaction loop
    async run(flags) {
        this.grokConfig = await loadGrokConfig(this.configDir);
        await this.initializeConversation(flags);
        await this.setupChatPage(flags.visible);
        if (flags.files)
            await this.uploadFiles(flags.files);
        console.log(chalk.blue('Interactive chat started. Type ') + chalk.yellow(':') + chalk.blue(' for menu, ') + chalk.yellow('\\q') + chalk.blue(' to exit'));
        await this.chatLoop(flags.visible);
    }
    async initializeConversation(flags) {
        if (flags.new) {
            this.conversationId = '';
            console.log(chalk.yellow('Starting a new conversation (ignoring active ID)'));
        }
        else {
            this.conversationId = flags.conversation || this.grokConfig.activeConversationId || '';
            if (this.conversationId && this.conversationId !== this.grokConfig.activeConversationId) {
                this.grokConfig.activeConversationId = this.conversationId;
                await saveGrokConfig(this.configDir, this.grokConfig);
                console.log(chalk.yellow(`Active conversation set to: ${this.conversationId}`));
            }
        }
    }
    async setupChatPage(visible) {
        const url = this.conversationId ? `https://grok.com/chat/${this.conversationId}` : 'https://grok.com';
        await this.page.goto(url, { waitUntil: 'networkidle2' });
        try {
            await this.page.waitForSelector('textarea[aria-label="Ask Grok anything"]', { timeout: 30000 });
        }
        catch (error) {
            console.log(chalk.red('Timed out waiting for chat UI'));
            const content = await this.page.content();
            console.log(chalk.yellow('Page content after timeout:'), content);
            throw new Error('Failed to load chat UI');
        }
        if (this.conversationId) {
            const spinner = ora('Loading previous conversation...').start();
            try {
                this.initialParentResponseId = (await this.conversationManager.loadConversationHistory(this.conversationId)) || '';
                if (this.initialParentResponseId) {
                    this.chatStreamer = new ChatStreamer(this.page, this.conversationId, this.initialParentResponseId);
                    spinner.succeed(chalk.green(`Loaded conversation ${this.conversationId}`));
                }
                else {
                    spinner.warn(chalk.yellow(`No history found for conversation ${this.conversationId}`));
                }
            }
            catch (error) {
                spinner.fail(chalk.red(`Failed to load conversation: ${error.message}`));
            }
        }
        else {
            this.chatStreamer = new ChatStreamer(this.page, this.conversationId, this.initialParentResponseId);
        }
    }
    async chatLoop(verbose) {
        this.chatActive = true;
        while (this.chatActive) {
            console.log(this.formatCwd());
            const userInput = await input({ message: chalk.green('You:') });
            if (userInput.trim() === '\\q') {
                this.chatActive = false;
            }
            else if (userInput.trim() === ':') {
                await this.menuHandler.showMenu(this);
            }
            else if (userInput.trim().startsWith(':')) {
                await this.commandRegistry.execute(userInput.trim(), this);
            }
            else {
                await this.chatStreamer.streamChat(userInput, this.fileIds, this.enabledTool, verbose);
                await this.updateConversationId();
                this.fileIds = [];
            }
        }
        console.log(chalk.blue('Chat session ended'));
    }
    async uploadFiles(globPattern) {
        const paths = globPattern.split(',').map(p => resolve(this.cwd, p));
        await this.fileUploader.uploadFiles(paths);
    }
    async updateConversationId() {
        if (!this.conversationId && this.chatStreamer.getConversationId()) {
            this.conversationId = this.chatStreamer.getConversationId();
            this.grokConfig.activeConversationId = this.conversationId;
            await saveGrokConfig(this.configDir, this.grokConfig);
        }
    }
    formatCwd() {
        const home = homedir();
        return this.cwd.startsWith(home) ? chalk.gray(`📁 ~${this.cwd.slice(home.length)}`) : chalk.gray(`📁 ${this.cwd}`);
    }
    getCwd() { return this.cwd; }
    setCwd(cwd) { this.cwd = cwd; }
    getEnabledTool() { return this.enabledTool; }
    setEnabledTool(tool) { this.enabledTool = tool; }
    getFileUploader() { return this.fileUploader; }
    getChatStreamer() { return this.chatStreamer; }
    getGrokConfig() { return this.grokConfig; }
}
