import { JSONParser } from '@streamparser/json-node';
import chalk from 'chalk';
import ora from 'ora';
import { Writable } from 'stream';
// Manages real-time streaming of chat responses from the Grok API
export class ChatStreamer {
    page;
    lastResponseId;
    conversationId;
    constructor(page, conversationId, initialParentResponseId = '') {
        this.page = page;
        this.conversationId = conversationId;
        this.lastResponseId = initialParentResponseId;
    }
    // Streams chat responses, handling tokens and applying tools if specified
    async streamChat(userInput, fileIds, tool, verbose = false) {
        const spinner = ora('Grok is thinking...').start();
        try {
            let processedPrompt = userInput;
            if (tool)
                processedPrompt = await tool.preprocessPrompt(processedPrompt);
            const payload = {
                temporary: false,
                modelName: 'grok-3',
                message: processedPrompt,
                fileAttachments: fileIds.slice(0, 10),
                imageAttachments: [],
                disableSearch: false,
                enableImageGeneration: true,
                returnImageBytes: false,
                enableImageStreaming: true,
                imageGenerationCount: 2,
                forceConcise: false,
                toolOverrides: {},
                enableSideBySide: true,
                sendFinalMetadata: true,
                isReasoning: false,
                webpageUrls: [],
                disableTextFollowUps: true,
                parentResponseId: this.lastResponseId
            };
            let fullResponse = '';
            let spinnerStopped = false;
            let modelResponse = null;
            const endpoint = this.conversationId
                ? `https://grok.com/rest/app-chat/conversations/${this.conversationId}/responses`
                : 'https://grok.com/rest/app-chat/conversations/new';
            const responsePromise = new Promise((resolve) => {
                const timeout = setTimeout(() => resolve('Error: Response timeout after 15 minutes'), 900000);
                const parser = new JSONParser({ paths: ['$.result', '$.error'], keepStack: false, separator: '' });
                const chunkWriter = new Writable({
                    write(chunk, encoding, callback) {
                        const text = chunk.toString();
                        if (text.startsWith('STREAM_CHUNK:')) {
                            parser.write(text.substring(13));
                        }
                        else if (text === 'STREAM_END') {
                            parser.end();
                        }
                        else if (text.startsWith('STREAM_ERROR:')) {
                            console.log(chalk.red(`Error: ${text.substring(13)}`));
                            resolve('');
                        }
                        callback();
                    }
                });
                parser.on('data', ({ value }) => {
                    if (value && 'code' in value && 'message' in value) {
                        console.log(chalk.red(`Error: ${JSON.stringify(value)}`));
                        resolve('');
                        return;
                    }
                    if (!this.conversationId && value?.conversation?.conversationId) {
                        this.conversationId = value.conversation.conversationId;
                    }
                    const token = value?.token || value?.response?.token;
                    if (token && typeof token === 'string' && token !== '') {
                        if (!spinnerStopped) {
                            spinner.stop();
                            spinnerStopped = true;
                        }
                        fullResponse += token;
                        if (verbose && tool) {
                            process.stdout.write(chalk.gray(token));
                        }
                        else if (!tool) {
                            process.stdout.write(chalk.white(token));
                        }
                    }
                    else {
                        const mr = value?.modelResponse || value?.response?.modelResponse;
                        if (mr) {
                            if (!spinnerStopped) {
                                spinner.stop();
                                spinnerStopped = true;
                            }
                            fullResponse = mr.message;
                            this.lastResponseId = mr.responseId;
                            modelResponse = mr;
                            if (!tool)
                                process.stdout.write(chalk.white(fullResponse));
                        }
                    }
                });
                parser.on('error', (error) => console.log(chalk.yellow(`Stream parse error: ${error.message}`)));
                parser.on('end', () => {
                    clearTimeout(timeout);
                    this.page.off('console', handleConsole);
                    resolve(fullResponse);
                });
                const handleConsole = (msg) => chunkWriter.write(msg.text());
                this.page.on('console', handleConsole);
                this.page.evaluate(async (url, body) => {
                    const response = await fetch(url, {
                        method: 'POST',
                        headers: {
                            'Content-Type': 'application/json',
                            'Accept': '*/*',
                            'Origin': 'https://grok.com',
                            'Referer': 'https://grok.com/'
                        },
                        body: JSON.stringify(body)
                    });
                    if (!response.body) {
                        console.log('STREAM_ERROR: No response body');
                        return;
                    }
                    const reader = response.body.getReader();
                    const decoder = new TextDecoder();
                    while (true) {
                        const { done, value } = await reader.read();
                        if (done) {
                            console.log('STREAM_END');
                            break;
                        }
                        const chunk = decoder.decode(value, { stream: true });
                        console.log('STREAM_CHUNK:' + chunk);
                    }
                }, endpoint, payload);
            });
            let finalOutput = await responsePromise;
            if (tool && modelResponse)
                finalOutput = await tool.postprocessResponse(modelResponse);
            if (!spinnerStopped)
                spinner.stop();
            process.stdout.write('\n');
            return finalOutput;
        }
        catch (error) {
            spinner.fail(chalk.red(`Chat error: ${error.message}`));
            return '';
        }
    }
    getLastResponseId() { return this.lastResponseId; }
    getConversationId() { return this.conversationId; }
}
